const express = require('express');

const idhmApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const squel = require('squel');

const query = require(`${libs}/middlewares/query`);

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const id2str = require(`${libs}/middlewares/id2str`);

let rqf = new ReqQueryFields();

idhmApp.get('/year_range', (req, res, next) => {
    req.sql.from('adh_idh')
    .field('MIN(adh_idh.ano_censo)', 'start_year')
    .field('MAX(adh_idh.ano_censo)', 'end_year');
    next();
}, query, response('range'));

idhmApp.get('/IDHM_level', (req, res, next) => {
    req.result = [
        {id: null, name: 'Não Classificado'},
        {id: 1, name: 'Muito Baixa'},
        {id: 2, name: 'Baixo'},
        {id: 3, name: 'Médio'},
        {id: 4, name: 'Alto'},
        {id: 5, name: 'Muito Alto'}
    ];
    next();
}, response('IDHM_level'));


rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'city',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'city_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'municipio_id',
        table: 'adh_idh'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'adh_idh'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'estado_id',
        table: 'adh_idh_uf'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'adh_idh_uf'
    }
  }).addValue({
      name: 'min_year',
      table: 'adh_idh',
      tableField: 'ano_censo',
      resultField: 'year',
      where: {
          relation: '>=',
          type: 'integer',
          field: 'ano_censo'
      }
  }).addValue({
      name: 'max_year',
      table: 'adh_idh',
      tableField: 'ano_censo',
      resultField: 'year',
      where: {
          relation: '<=',
          type: 'integer',
          field: 'ano_censo'
      }
});


idhmApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
  log.debug(req.sql.toParam());
  if(typeof req.filter === 'undefined' || Object.keys(req.filter).length === 0 ) {
    res.status(400);
    next({
        status: 400,
        message: 'Wrong/No filter specified'
    });
  }
  if ("state" in req.filter) {
      req.sql.field('(adh_idh_uf.idhm)', 'IDHM')
     .field("'Brasil'", 'name')
     .field('adh_idh_uf.estado_id', 'estado_id')
     .field('adh_idh_uf.ano_censo', 'year')
     .from('adh_idh_uf')
   } else if ("city" in req.filter) {
     req.sql.field('(adh_idh.idhm)', 'IDHM')
    .field("'Brasil'", 'name')
    .field('adh_idh.municipio_id', 'municipio_id')
    .field('adh_idh.estado_id', 'estado_id')
    .field('adh_idh.ano_censo', 'year')
    .from('adh_idh')
  }
  next();
}, query, id2str.transform(true), response('idhm'));

module.exports = idhmApp;
